# -*- coding: utf-8 -*-

import string
import random

from context import *

# TODO: Mock mysql
def random_letter(len):
    letters = string.ascii_letters
    return ''.join(random.choice(letters) for _ in range(len))


def random_letter_num(len):
    letters = string.ascii_letters + string.digits + '\''
    return ''.join(random.choice(letters) for _ in range(len))


def test():
    dumper = autodump.Autodumper(random_letter(10))
    n_test, n_col = 10, 10
    col_name_len, record_len = 10, 5
    col_name = [random_letter(col_name_len) for _ in range(col_name_len)]
    dumper.persist(random_letter_num(10), random_letter_num(10))
    for t in range(n_test):
        record = []
        for c in range(n_col):
            record.append(random_letter_num(record_len))
            dumper.cache(col_name[c], record[-1])
        print("Test %d: %s" % (t, record))
        dumper.flush()


if __name__ == "__main__":
    test()



# -*- coding: utf-8 -*-

from autodump.main import *
from mysql import connector
import mysql.connector.pooling
import logging

from .common import *


class MysqlMedium(Medium):
    POOL_SIZE = 10
    REQUIRED_PARAM = ["host", "user", "port", "password", "database"]

    def __init__(self, space, param):
        super(MysqlMedium, self).__init__(param)
        self._table = space
        self._source = [None]
        self._reconnect()
        if not self._table_exists(self._table):
            self._new_table(self._table)
        self._cache = {}
        self._persistence = {}

    # TODO: Add and flush asynchronously, a good choice is using message queue
    def cache(self, k, v):
        self._cache[k] = v

    def flush(self):
        cache = {**self._persistence, **self._cache}
        for k in cache:
            if not self._column_exists(k):
                self._new_column(k)
        self._insert_cache(cache)

    def persist(self, k, v):
        self._persistence[k] = v

    def _check_param(self):
        if sorted(MysqlMedium.REQUIRED_PARAM) != sorted(list(self._param.keys())):
            raise Exception(
                "Invalid param, required param is: %s, your input is: %s" % (
                    MysqlMedium.REQUIRED_PARAM, self._param.keys()))

    def _reconnect(self):
        source = connector.pooling.MySQLConnectionPool(
            pool_name="pool.%s.%s" % (self._param["host"], self._param["database"]),
            pool_size=MysqlMedium.POOL_SIZE,
            **self._param)
        self._source[0] = source

    def activate(self):
        self._reconnect()

    # TODO: 1. Add INT and DOUBLE type, save priority: INT, DOUBLE, VARCHAR(255)
    #       2. Persistent kv: insert every time
    def _insert_cache(self, cache):
        SQL = ("INSERT INTO {0} ({1}) VALUES (" + (', '.join((["%s"]*len(cache.keys())))) + ")").format(self._table, (", ".join([str(key) for key in cache.keys()])))
        value = list(cache.values())
        self._execute(SQL, value)
        self._cache.clear()

    def _column_exists(self, k):
        SQL = "SHOW COLUMNS FROM {0} LIKE %s".format(self._table)
        value = [k]
        rowcount = self._execute(SQL, value)
        return rowcount != 0

    def _new_column(self, k):
        # Can not use placeholder for table names
        SQL = "ALTER TABLE {0} ADD {1} VARCHAR(1023)".format(self._table, k)
        value = []
        return self._execute(SQL, value)

    def _table_exists(self, table):
        SQL = "SHOW TABLES LIKE %s"
        value = [table]
        rowcount = self._execute(SQL, value)
        return rowcount != 0

    def _new_table(self, table):
        SQL = "CREATE TABLE {0} (id INT NOT NULL AUTO_INCREMENT, time DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP, PRIMARY KEY (id))".format(table)
        value = []
        return self._execute(SQL, value)

    # TODO: Handle Exception
    def _execute(self, SQL, value):
        connection = self._source[0].get_connection()
        cursor = connection.cursor()
        # TODO: See if there is better way
        value = [str(_value).replace('\'', '\'\'') for _value in value]
        cursor.execute(SQL, value)
        cursor.fetchall()
        rowcount = cursor.rowcount
        connection.commit()
        connection.close()
        return rowcount



from abc import ABC, abstractmethod


class Medium(ABC):

    def __init__(self, param):
        self._param = param
        self._check_param()

    @abstractmethod
    def activate(self):
        pass

    @abstractmethod
    def _check_param(self):
        pass

    @abstractmethod
    def cache(self):
        pass

    @abstractmethod
    def flush(self):
        pass

    @abstractmethod
    def persist(self):
        pass



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="autodump",
    version="1.0.0",
    author="Ping Z.",
    author_email="ping69852@gmail.com",
    description="Automatically and non-invasively dump your (experiment) data",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/noilreed/autodump",
    project_urls={
        "Bug Tracker": "https://github.com/noilreed/autodump/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(exclude=["test"]),
    install_requires=['mysql-connector-python >= 8.0.25', 'PyYAML'],
    python_requires=">=3.6",
)



# -*- coding: utf-8 -*-

import inspect
import os
import yaml

from .medium import *


class Autodumper:
    def __init__(self, space, config=None, config_path="autodump.yaml"):
        if not ((config_path is None) ^ (config is None)):
            raise Exception("Either config or config_path (and not all) should be specified. ")
        elif config_path is not None:
            if os.path.isabs(config_path):
                config = yaml.safe_load(config_path)
            else:
                caller_file = inspect.stack()[1][0].f_code.co_filename
                config_path = os.path.join(os.path.split(caller_file)[0], config_path)
                config = yaml.safe_load(open(config_path))
        self._config = config
        self._medium = self._init_medium(space)
        self._medium.activate()

    def persist(self, k, v):
        self._medium.persist(k, v)

    def cache(self, k, v):
        self._medium.cache(k, v)

    def flush(self):
        self._medium.flush()

    def _init_medium(self, space):
        param, medium = self._config["param"], self._config["medium"]
        if medium == "mysql":
            medium = MysqlMedium(space, param)
        else:
            raise Exception("Medium type %s not supported. " % medium)
        return medium
